import { App } from 'vue';
import { InjectionKey } from 'vue';

export declare type BindingObject = {
    [key: string]: any;
};

export declare const CheckboxField: ({ value, ...rest }: Partial<CheckboxInput>) => CheckboxInput;

export declare type CheckboxInput = InputBase & {
    type: FieldTypes.CHECKBOX;
    value: boolean;
};

export declare const ColorField: ({ value, ...rest }: Partial<ColorInput>) => ColorInput;

export declare type ColorInput = InputBase & {
    type: FieldTypes.COLOR;
    value: string;
};

export declare function createDynamicForms(options?: DynamicFormsOptions): DynamicFormsPlugin;

export declare const CustomField: ({ value, ...rest }: Partial<CustomInput>) => CustomInput;

export declare type CustomInput = InputBase & {
    type: FieldTypes.CUSTOM;
    value: boolean | string | number;
};

export declare interface DynamicFormsOptions {
    autoValidate?: boolean;
    form?: FormOptions;
}

export declare interface DynamicFormsPlugin {
    options?: DynamicFormsOptions;
    install(app: App): void;
}

export declare const dynamicFormsSymbol: InjectionKey<DynamicFormsPlugin>;

export declare const email: (control: FormControl<InputType>) => ValidationErrors | null;

export declare const EmailField: ({ value, ...rest }: Partial<EmailInput>) => EmailInput;

export declare type EmailInput = InputBase & {
    type: FieldTypes.EMAIL;
    value: string;
};

export declare const FieldBase: ({ validations, label, ariaLabel, ariaLabelledBy, customClass, customStyles, disabled, placeholder, required, autocomplete, readonly, }: InputBase) => InputBase;

export declare const enum FieldTypes {
    TEXT = "text",
    TEXTAREA = "textarea",
    SELECT = "select",
    NUMBER = "number",
    EMAIL = "email",
    URL = "url",
    PASSWORD = "password",
    CHECKBOX = "checkbox",
    RADIO = "radio",
    CUSTOM = "custom-field",
    COLOR = "color"
}

export declare type FormControl<T extends InputType> = T & {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
};

export declare interface FormOptions {
    customClass?: string | string[] | BindingObject | BindingObject[];
    customStyles?: string | string[] | BindingObject | BindingObject[];
    method?: string;
    netlify?: boolean;
    netlifyHoneypot?: string;
    autocomplete?: boolean;
}

export declare interface FormValidation {
    validator: ValidatorFn;
    text: string;
}

export declare interface InputBase {
    name?: string;
    label?: string;
    ariaLabel?: string;
    ariaLabelledBy?: string;
    required?: boolean;
    disabled?: boolean;
    customClass?: string | string[] | BindingObject | BindingObject[] | unknown;
    customStyles?: string | string[] | BindingObject | BindingObject[] | unknown;
    placeholder?: string;
    autocomplete?: string;
    readonly?: boolean;
    validations?: FormValidation[];
}

export declare type InputType = TextInput | NumberInput | SelectInput | TextAreaInput | CheckboxInput | RadioInput | EmailInput | PasswordInput | RadioInput | ColorInput | UrlInput | CustomInput;

export declare const isEmptyInputValue: (value: string | number | boolean) => boolean;

export declare const max: (max: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const maxLength: (maxLength: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const min: (min: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const minLength: (minLength: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare type MustHave<T, K extends keyof T> = Partial<T> & Pick<T, K>;

export declare const NumberField: ({ value, min, max, step, ...rest }: Partial<NumberInput>) => NumberInput;

export declare type NumberInput = InputBase & {
    type: FieldTypes.NUMBER;
    value: number;
    min: number;
    max: number;
    step: number;
};

export declare const PasswordField: ({ value, ...rest }: Partial<PasswordInput>) => PasswordInput;

export declare type PasswordInput = InputBase & {
    type: FieldTypes.PASSWORD;
    value: string;
};

export declare const pattern: (pattern: string) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const RadioField: ({ options, value, ...rest }: Partial<RadioInput>) => RadioInput;

export declare type RadioInput = InputBase & {
    type: FieldTypes.RADIO;
    value: string;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare const required: (control: FormControl<InputType>) => ValidationErrors | null;

export declare const SelectField: ({ options, value, ...rest }: Partial<SelectInput>) => SelectInput;

export declare type SelectInput<T = boolean | string> = InputBase & {
    type: FieldTypes.SELECT;
    value: T;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare type TextAreaInput = InputBase & {
    type: FieldTypes.TEXTAREA;
    value: string;
    cols?: number;
    rows?: number;
};

export declare const TextField: ({ value, ...rest }: Partial<TextInput>) => TextInput;

export declare type TextInput = InputBase & {
    type: FieldTypes.TEXT;
    value?: string;
};

export declare const url: (control: FormControl<InputType>) => ValidationErrors | null;

export declare type UrlInput = InputBase & {
    type: FieldTypes.URL;
    value: string;
};

export declare function useDynamicForms(): DynamicFormsPlugin;

export declare type ValidationErrors = {
    [key: string]: any;
};

declare interface ValidatorFn {
    (control: FormControl<InputType> | undefined): ValidationErrors | null;
}

export { }
